/*
 *      pehdr.h
 *
 *      Copyright (c) 1994 by Borland International
 *      All Rights Reserved.
 *
 */

#ifndef __PEHDR_H__
#define __PEHDR_H__

#define PESIGNATURE		0x4550

typedef struct {

    bits8       signature [4];
    bits16      CPUType;
    bits16      nObjects;
    bits32      timeDate;
    bits32      symbolPtr;      // reserved in PE spec
    bits32      nSymbols;       // reserved in PE spec
    bits16      optHeaderSize;  // reserved in PE spec
    bits16      flags;
    bits16      magicNumber;    // reserved in PE spec
    bits8       LMajor,         LMinor;
    bits32      codeSize;       // reserved in PE spec
    bits32      initDataSize;   // reserved in PE spec
    bits32      uninitDataSize; // reserved in PE spec
    bits32      entryRVA;
    bits32      codeBase;       // reserved in PE spec
    bits32      dataBase;       // reserved in PE spec
    bits32      imageBase;
    bits32      objectAlign;
    bits32      fileAlign;
    bits16      OSMajor,        OSMinor;
    bits16      userMajor,      userMinor;
    bits16      subsysMajor,    subsysMinor;
    bits32      reserved6;
    bits32      imageSize;
    bits32      headerSize;
    bits32      chksum;
    bits16      subSystem;
    bits16      dllFlags;
    bits32      stackReserve,   stackCommit;
    bits32      heapReserve,    heapCommit;
    bits32      reserved7;
    bits32      nInterestingRVAs;
    bits32      exportRVA,      exportSize;
    bits32      importRVA,      importSize;
    bits32      resourceRVA,    resourceSize;
    bits32      exceptionRVA,   exceptionSize;
    bits32      securityRVA,    securitySize;
    bits32      fixupRVA,       fixupSize;
    bits32      debugRVA,       debugDirCount;
    bits32      descriptionRVA, descriptionSize;
    bits32      MSRVA,          MSSize;
	bits32		TLSRVA,			TLSSize;
	bits32		callbackRVA,	callbackSize;
    bits32      reserved8 [10];
} PEHeader;

typedef struct {
    bits32      flags;
    bits32      timeDate;
    bits16      userMajor,      userMinor;
} BasicDirectory;

typedef struct {
	BasicDirectory	basic;
    bits32      nameRVA;
    bits32      ordinalBase;
    bits32      nEntries;
    bits32      nNamePtrs;
    bits32      addrRVA;
    bits32      namePtrRVA;
    bits32      ordinalRVA;
} ExportDirectory;

typedef struct {
	BasicDirectory	basic;
    bits32      nameRVA;
	bits32		lookupRVA;
} ImportDirectory;

typedef struct {
    bits32      pageRVA;
    bits32      size;
} FixupBlock;

typedef struct {
	BasicDirectory	basic;
    bits32      type;
    bits32      size;
    bits32      RVA;
    bits32      seek;
} DebugDirectory;

typedef struct {
    bits32   NumberOfSymbols;
    bits32   LvaToFirstSymbol;
    bits32   NumberOfLinenumbers;
    bits32   LvaToFirstLinenumber;
    bits32   RvaToFirstByteOfCode;
    bits32   RvaToLastByteOfCode;
    bits32   RvaToFirstByteOfData;
    bits32   RvaToLastByteOfData;
} DebugInfo;

typedef struct {
    bits8       name [8];
    bits32      virtualSize;
    bits32      RVA;
    bits32      physicalSize;
    bits32      physicalOffset;
    bits32      reserved [3];
    bits32      flags;
} PEObjectTabEntry;

#define PEFIXUP_MASK            0xf000
#define PEFIXUP_ABSOLUTE        0x0000
#define PEFIXUP_HIGH            0x1000
#define PEFIXUP_LOW             0x2000
#define PEFIXUP_HIGHLOW         0x3000
#define PEFIXUP_HIGHADJUST      0x4000
#define PEFIXUP_MIPSJMPADDR     0x5000

#define F_PEOBJ_CODE            0x00000020
#define F_PEOBJ_INITDATA        0x00000040
#define F_PEOBJ_UNINITDATA      0x00000080
#define F_PEOBJ_NOCACHE         0x04000000
#define F_PEOBJ_NOPAGE          0x08000000
#define F_PEOBJ_SHARED          0x10000000
#define F_PEOBJ_E               0x20000000
#define F_PEOBJ_R               0x40000000
#define F_PEOBJ_W               0x80000000

#define PECPU_UNKNOWN           0x0000
#define PECPU_80386             0x014c
#define PECPU_80486             0x014d
#define PECPU_80586             0x014e
#define PECPU_MIPSI             0x0162
#define PECPU_MIPSII            0x0163
#define PECPU_MIPSIII           0x0166

#define F_PE_IMAGE              0x0000
#define F_PE_FIXED              0x0001
#define F_PE_EXECUTABLE         0x0002
#define F_PE_NOLINENUM          0x0004
#define F_PE_NOSYMBOLS          0x0008
#define F_PE_BYTESREVERSED      0x0080
#define F_PE_32BIT              0x0100
#define F_PE_LIBRARY            0x2000
#define F_PE_MAGIC              0x8000

#define PE_SUBSYS_UNKNOWN       0x0000
#define PE_SUBSYS_NATIVE        0x0001
#define PE_SUBSYS_WINGUI        0x0002
#define PE_SUBSYS_WINCHARACTER  0x0003
#define PE_SUBSYS_OS2CHARACTER  0x0005
#define PE_SUBSYS_POSIXCHARACTER    0x0007

#define F_PE_DLL_PROCINIT       0x0001
#define F_PE_DLL_PROCTERM       0x0002
#define F_PE_DLL_THREADINIT     0x0004
#define F_PE_DLL_THREADTERM     0x0008

#endif

